function [sys,x0,str,ts] = columnbin(t,x,u,flag,n,f,aa,H,oxo)
%  columnbin(t,x,u,flag,n,f) simulates a binary distillation column
% with Lewis hypotheses: liquid and vapor flowrates along the column are
% constants
%
%  ---- PARAMETERS
%  n  :: number of trays of he column
%  f  :: index of the feed tray
%  aa :: relative volativity of the mixture (>1, close to 1)
%   H :: liquid hold-up on each tray
%
%  ---- Inputs
%   1. vapor flow rate (V) (control variable)
%   2. reflux flow rate (L) (control variable) 
%   3. vapor feed rate (FV)
%   4. liquid feed rate (FL) => F=FL+FV is the total feed rate
%   5. in-feed composition, molar fraction of light component in the feed (Zf)
%
%  ---- outputs
%  1. head tray light component composition (x1)  
%  2. Bottom tray light component composition (xn, yn=k(xn) see model)

% Last modified by Elnycko 26/07/2007

switch flag,
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes(n,H,oxo);
  case 1,
    sys=mdlDerivatives(t,x,u,n,f,aa,H);
  case 3,
    sys=mdlOutputs(t,x,u,n);
  case {4,2,9}
    sys=[];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);
end

function [sys,x0,str,ts]=mdlInitializeSizes(n,H,oxo)
sizes = simsizes;
sizes.NumContStates  = n;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = n;
sizes.NumInputs      = 5;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % at least one sample time is needed

sys = simsizes(sizes);
% x0  = zeros(n,1);
% x0=[    0.9948
%     0.9911
%     0.9863
%     0.9799
%     0.9714
%     0.9603
%     0.9460
%     0.9277
%     0.9048
%     0.8766
%     0.8431
%     0.8042
%     0.7609
%     0.7145
%     0.6547
%     0.5775
%     0.4863
%     0.3884
%     0.2941
%     0.2119
%     0.1465
%     0.0980
%     0.0640
%     0.0410
%     0.0258
%     0.0160
%     0.0097];
x0=oxo;
str = [];
ts  = [0 0];
if length(H)~=n
    error(['parameter vector H should have length ', n]);
end

% end mdlInitializeSizes

function sys=mdlDerivatives(t,x,u,n,f,aa,H)
V=u(1);
L=u(2);
FV=u(3);
FL=u(4);
Zf=u(5);
if size(H,1)<size(H,2)
    H=H';
end,

G11=eye(f-1,f-1)*L-diag(ones(f-2,1),1)*L; % do not take the last line
G12=-eye(n-f+1,n-f+1)*(L+FL)+(L+FL)*diag(ones(n-f,1),-1);
G1=[-(V+FV), zeros(1,n-1);
    G11(1:f-2,:), zeros(f-2,n-f+1);
    zeros(1,f-2),L,-(L+FL),zeros(1,n-f);
    zeros(n-f,f-1),G12(2:n-f+1,:)];
G1(n,n)=G1(n,n)+V;

G21=diag(ones(f-1,1),1)*(V+FV)-eye(f,f)*(V+FV);
G21(1,1)=0;
G22=diag(ones(n-f,1)*V,1)-eye(n-f+1,n-f+1)*V;
G2=[G21(1:f-1,:),zeros(f-1,n-f);
    zeros(1,f-1),-(FV+V),V,zeros(1,n-f-1);
    zeros(n-f,f),G22(2:n-f+1,2:n-f+1)];

G3=zeros(n,1);
G3(f)=FL*Zf+FV*k(Zf,aa);
sys = (G1*x+G2*k(x,aa)+G3)./H;
% end mdlDerivatives

function sys=mdlOutputs(t,x,u,n)

sys = x;
% end mdlOutputs

function y=k(x,aa)
y1=aa*x;
y2=1+(aa-1)*x;
y=y1./y2;
