function [sys,x0,str,ts] = voyez_colombine(t,x,u,flag,n,f,aa,H,Q,R,theta,oxo)
%  voyez_colombine(t,x,u,flag,n,f,aa,H,Q,R)
% is the high-gain extended Kalman filter for the binary distillation
% column (n trays, feed at tray f)
%
%  ---- PARAMETERS
%   n :: number of trays in the column
%   f :: index of the feeding tray
%  aa :: relative volativity of the mixture (>1, close to 1)
%   H :: liquid hold-up on each tray
%  Q and R :: the weigth matrices of the riccati equation
%
%  ---- Inputs
%   1. vapor flow rate (V) (control variable)
%   2. reflux flow rate (L) (control variable) 
%   3. vapor feed rate (FV)
%   4. liquid feed rate (FL) => F=FL+FV is the total feed rate
%   5. in-feed composition, molar fraction of light component in the feed (Zf)
%   6. x1
%   7. x_27
%
%  ---- outputs
%  x_1 -> x_27

% Last modified by Elnycko 26/07/2007

switch flag,
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes(H,n,f,aa,oxo);
  case 1,
    sys=mdlDerivatives(t,x,u,aa,H,Q,R,n,f,theta);
  case 3,
    sys=mdlOutputs(t,x,u,n,f,aa);
  case {4,2,9}
    sys=[];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);
end

function [sys,x0,str,ts]=mdlInitializeSizes(H,n,f,aa,oxo)
sizes = simsizes;
sizes.NumContStates  = n+n*(n+1)/2;  % state + riccati
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = n;
sizes.NumInputs      = 5+2;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % at least one sample time is needed

sys = simsizes(sizes);
% c0=[0.7:-0.6/(n-1):0.1]';

% c0=[    0.9948
%     0.9911
%     0.9863
%     0.9799
%     0.9714
%     0.9603
%     0.9460
%     0.9277
%     0.9048
%     0.8766
%     0.8431
%     0.8042
%     0.7609
%     0.7145
%     0.6547
%     0.5775
%     0.4863
%     0.3884
%     0.2941
%     0.2119
%     0.1465
%     0.0980
%     0.0640
%     0.0410
%     0.0258
%     0.0160
%     0.0097];
c0=oxo;
K=Phi(c0(2:f-1),aa);
Kp=PhiPrime(c0(2:f-2),aa);  % k'2 -> k'f-2
bloc=toeplitz([1;Kp],ones(1,f-1));
bloc=prod(bloc,2); % products along the lines (result is column vector)
z0 = [c0(1);bloc.*K;c0(n:-1:f)];
p0=SymReshape(eye(n,n));
x0 =[z0;p0]; 
str = [];
ts  = [0 0];
if length(H)~=n
    error(['parameter vector H doesn''t have the proper length ']);
end
% end mdlInitializeSizes

function sys=mdlDerivatives(t,x,u,aa,H,Q,R,n,f,theta)
% -- parameters
rica=n*(n+1)/2;
V=u(1);
L=u(2);
FV=u(3);
FL=u(4);
Zf=u(5);
Y=[u(6);u(7)];
if size(H,1)<size(H,2)
    H=H';
end,
P=SymReshape(x(n+(1:rica)));
z=x(1:n); 
zprime=zeros(n,1);
% used for the computation of the jacobian matrix
zstep=zeros(n,1);
zStepPrime=zeros(n,1);
step=0.0001;

% -- A, C and b* matrices (z coordinates)
A=[zeros(n,1),[diag((ones(f-2,1)*(V+FV))./H(1:f-2));zeros(n-f+2,f-2)],zeros(n,1), [zeros(f-1,n-f); diag((ones(n-f,1)*(L+FL))./H(n:-1:f+1));zeros(1,n-f)]];
C=[1, zeros(1,n-1); zeros(1,f-1),1,zeros(1,n-f)];
bstar=zeros(n,n);

%%%  Luenberger approach
% gain=[    2.8608    0.0000
%     9.5788    0.0000
%     4.4897    0.0000
%     1.0000    0.0000
%     0.0000    3.3075
%     0.0000   26.5048
%     0.0000   15.8494
%     0.0000    5.7183
%     0.0000    1.0000];
% %----%



Delta=diag((theta*ones(1,n)).^[1:f-1,0:f-1]);
Q=theta^2*Delta*Q*Delta;

% -- From z to  x coordinates
xReturn=[z(1);zeros(f-2,1);z(n:-1:f)]; 
%     bin=(xReturn>1);
%     xReturn(bin)=1;
%     bin=(xReturn<0);
%     xReturn(bin)=0;
D=1;
for i=2:(f-1)
    xReturn(i)=PhiInv(z(i)/D,aa);    
%     if (xReturn(i)>1), xReturn(i)=1; end,
%     if (xReturn(i)<0), xReturn(i)=0; end,
    D=D*PhiPrime(xReturn(i),aa);
end,
Kn=Phi(xReturn(2:(f-1)),aa);   %k(x_2->x_f-1) :: dim f-1
Kp=PhiPrime(xReturn(2:(f-1)),aa);%k'(x_2->x_f-1) :: dim f-1
Kpp=PhiSecond(xReturn(2:(f-2)),aa);  %k''x_2->x_f-2) :: dim f-2

%  -- computation of the derivative in the x coordinates
G11=eye(f-1,f-1)*L-diag(ones(f-2,1),1)*L; % do not take the last line
G12=-eye(n-f+1,n-f+1)*(L+FL)+(L+FL)*diag(ones(n-f,1),-1);
G1=[-(V+FV), zeros(1,n-1);
    G11(1:f-2,:), zeros(f-2,n-f+1);
    zeros(1,f-2),L,-(L+FL),zeros(1,n-f);
    zeros(n-f,f-1),G12(2:n-f+1,:)];
G1(n,n)=G1(n,n)+V;
G21=diag(ones(f-1,1),1)*(V+FV)-eye(f,f)*(V+FV);
G21(1,1)=0;
G22=diag(ones(n-f,1)*V,1)-eye(n-f+1,n-f+1)*V;
G2=[G21(1:f-1,:),zeros(f-1,n-f);
    zeros(1,f-1),-(FV+V),V,zeros(1,n-f-1);
    zeros(n-f,f),G22(2:n-f+1,2:n-f+1)];  
G3=zeros(n,1);
G3(f)=FL*Zf+FV*aa*Zf/(1+(aa-1)*Zf);
xprime =(G1*xReturn+G2*Phi(xReturn,aa)+G3)./H;  
% here I have the results for the derivatives in the X coordinates 
% we only need to compute z(2->f-1) now

% computation of z(1->f-1)
Fin=prod(toeplitz(Kp,[Kp(1),ones(1,f-3)]),2).*xprime(2:f-1);  % !! Fin va de 2  f-1

zprime(1)=xprime(1); %(V+FV)/H(1)*(z(2)-z(1));   
zprime(2)=Fin(1); % Fin commence avec k'(x_2).(dx2/dt)

if f>3
    for k=3:f-1
        bloc=Kp(1:(k-2))*ones(1,k-2);
        bloc=prod(bloc-diag(diag(bloc))+eye(k-2,k-2),1)';     % ..'.. is there to obtain a column vector
        bloc=bloc.*Kpp(1:(k-2)).*xprime(2:k-1);
        zprime(k)=sum(bloc)*Kn(k-1)+Fin(k-1);                   % Note K(k-1) is k(x_k), but xprime(k-1)=d(x_k-1)/dt
    end,
end, 
% Now I have the values of the derivatives for Z_1 -> Z_{f-1}
zprime(f:n)=xprime(n:-1:f);
% I now have the values of ALL the derivatives in the z coordinates !
%AND THEN
sys(1:n)=zprime-P*C'*inv(R)*(C*z-Y); %Kalman like
% sys(1:n)=zprime-Delta*gain*(C*z-Y); %lunberger like


% -------  Riccati equation  -------
%maybe this will be a little bit too long but we will compute the Jacobian
%matrix of b without using the explicit formulas and a step of length
%0.0001 (1O^-4)
bstar(1,1)=-(V+FV)/H(1);      % b11
bstar(2,1)=L/H(2)*Kp(1);  % b21
bstar(3:f-1,1)=L*Kpp(1)*prod(toeplitz([1;Kp(2:f-3)],ones(1,f-3)),2).*Kn(2:f-2)./H(3:f-1);

for m=2:n  % browse the columns of the jacobian matrix
    zstep=z;
    zstep(m)=zstep(m)+step; %only the mth variable is incremented (i think there are smartest ways to do this)
    xStepReturn=[zstep(1);zeros(f-2,1);zstep(n:-1:f)];
    D=1;        % from zstep -> xstep
    for i=2:(f-1)   % From Z to X
        xStepReturn(i)=PhiInv(zstep(i)/D,aa);
        D=D*PhiPrime(xStepReturn(i),aa);
    end,
        
    Kn(:)=Phi(xStepReturn(2:(f-1)),aa);   %k(x_2->x_f-1) :: dim f-1
    Kp(:)=PhiPrime(xStepReturn(2:(f-1)),aa);  %k'(x_2->x_f-1) :: dim f-1
    Kpp(:)=PhiSecond(xStepReturn(2:(f-2)),aa);  %k''x_2->x_f-2) :: dim f-2
    
    xStepPrime =(G1*xStepReturn+G2*Phi(xStepReturn,aa)+G3)./H; %xdot value computed from Zstep
    Fin=prod(toeplitz(Kp,[Kp(1),ones(1,f-3)]),2).*xStepPrime(2:f-1);  % !! Fin va de 2  f-1
   
    zStepPrime(1)=xStepPrime(1);    %(V+FV)/H(1)*(zstep(2)-zstep(1));
    zStepPrime(2)=Fin(1); % Fin commence avec k'(x_2).(dx2/dt)

    if f>3
        for k=3:f-1
        bloc=Kp(1:(k-2))*ones(1,k-2);
        bloc=prod(bloc-diag(diag(bloc))+eye(k-2,k-2),1)';
        bloc=bloc.*Kpp(1:(k-2)).*xStepPrime(2:k-1);
        zStepPrime(k)=sum(bloc)*Kn(k-1)+Fin(k-1); % Note K(k-1) is k(x_k), but xprime(k-1)=d(x_k-1)/dt
        end,
    end,
  
    zStepPrime(f:n)=xStepPrime(n:-1:f);

    bregular=zprime-A*z;        % vector field b(z,u)
    bstep=zStepPrime-A*zstep;  % vector field b(z+dz,u)
    bstar(:,m)=(bstep-bregular)/step;    
end,

sys(n+(1:rica))=SymReshape(P*(A+bstar)'+(A+bstar)*P-P*C'*inv(R)*C*P+Q);
% end mdlDerivatives

function sys=mdlOutputs(t,x,u,n,f,aa)
z=x(1:n); 
xReturn=[z(1);zeros(f-2,1);z(n:-1:f)]; 
D=1;
for i=2:(f-1)
    xReturn(i)=PhiInv((z(i)/D),aa);    
    D=D*PhiPrime(xReturn(i),aa); 
end,
sys =xReturn;
% end mdlOutputs

function y=Phi(x,aa)
y1=aa*x;
y2=1+(aa-1)*x;
y3=1-(aa-1)*x;
y=(y1./y2); %.*(x>=0)+(y1./y3).*(x<0);

function y=PhiPrime(x,aa)
% derivative of the above function
y2=(1+(aa-1)*x).^2;
y3=(1-(aa-1)*x).^2;
y=(aa./y2); %.*(x>=0)+(aa./y3).*(x<0);

function y=PhiSecond(x,aa)
%second derivative of the same function
y1=2*aa*(1-aa);
y2=(1+(aa-1)*x).^3;
y3=(1-(aa-1)*x).^3;
y=(y1./y2); %.*(x>=0)  +(-y1./y3).*(x<0);

function y=PhiInv(x,aa)
y2=aa-x*(aa-1);
y3=aa-x*(aa-1);
y=(x./y2); %.*(x>=0)+(x./y3).*(x<0);;